use std::time::Duration;

use api::init_api;
use bot::start_bot;
use config::parse_local_config;
use cron_scheduler::ScheduleJob;
use database::{create_manager_and_init, create_pool_manager};
use tokio::{sync::oneshot, time::sleep};
use tool_tracing::init::init_tracing;
use tracing::{error, info};

#[tokio::main]
async fn main() -> Result<(), ()> {
    println!(include_str!("banner.art"));
    let config = parse_local_config();
    init_tracing(config.tracing.clone(), config.bot_name.clone());

    info!("Init Database");
    let datalake_config = create_pool_manager(config.persistence.clone()).unwrap();
    let manager = match create_manager_and_init(datalake_config).await {
        Ok(manager) => {
            info!("Database manager created successfully");
            manager
        }
        Err(e) => {
            error!("Failed to create database manager: {}", e);
            return Err(());
        }
    };
    info!("Database manager initialized successfully");
    let mut cron_scheduler = match ScheduleJob::start_cron_scheduler().await {
        Ok(scheduler) => {
            info!("Cron scheduler started successfully");
            scheduler
        }
        Err(_) => {
            error!("Failed to start cron scheduler");
            return Err(());
        }
    };
    let (tx_bot, rx_bot) = oneshot::channel();
    let http = start_bot(config.clone(), manager.clone(), rx_bot).await;

    let _ = init_api(config, manager, http).await;

    tx_bot.send(()).unwrap_or_else(|_| {
        error!("Failed to send shutdown signal to bot");
    });
    cron_scheduler.stop_cron_scheduler().await;
    sleep(Duration::from_secs(2)).await;
    //process::exit(1);
    Ok(())
}
