use clickhouse_pool::traits::Model;
use database::guild::Guild;
use tracing::{info, warn};

use crate::{Context, Error};

/// Check if the guild has been initialized, if it is check if user is in list of admins
/// If not, check the user's permissions
/// If the user is not allowed dont allow the action
pub async fn is_user_admin_right(ctx: Context<'_>) -> Result<bool, Error> {
    let guild_id = match ctx.guild_id() {
        Some(id) => id.get(),
        None => {
            ctx.say("This command can only be used in a server.")
                .await?;
            info!(
                "User {} tried to use a guild command outside of a guild.",
                ctx.author().id
            );
            return Ok(false);
        }
    };

    // Check if the user has the required permissions
    let users_permissions = match ctx.author_member().await.unwrap().permissions {
        Some(permissions) => permissions,
        None => {
            warn!("User permissions not found for user: {}", ctx.author().id);
            return Ok(false);
        }
    };
    if users_permissions.administrator()
        || users_permissions.manage_guild()
        || users_permissions.manage_roles()
        || users_permissions.manage_channels()
    {
        info!(
            "User {} has admin rights in guild {}.",
            ctx.author().id,
            guild_id
        );
        return Ok(true);
    }

    let manager_pool = ctx.data().datalake_config.clone();

    let fetch_query = Guild::build_select_query(Some(&format!("id = {}", guild_id)), None, None);
    let guilds = manager_pool
        .execute_select_with_retry::<Guild>(&fetch_query)
        .await?;
    if guilds.is_empty() {
        warn!("Guild with ID {} not found in database.", guild_id);
        return Ok(false);
    }
    let guild = &guilds[0];
    if guild.enrolled {
        if guild.moderator.contains(&ctx.author().id.get()) {
            info!(
                "User {} is a moderator in guild {}.",
                ctx.author().id,
                guild_id
            );
            return Ok(true);
        }
    }
    info!(
        "User {} does not have admin rights in guild {}.",
        ctx.author().id,
        guild_id
    );
    Ok(false)
}

// pub async fn autocomplete_channel_that_have_trivia(
//     ctx: Context<'_>,
//     _partial: &str,
// ) -> impl Iterator<Item = String> {
//     let guild_id = match ctx.guild_id() {
//         Some(id) => id.get(),
//         None => {
//             ctx.say("This command can only be used in a server.").await;
//             return vec![];
//         }
//     };

//     let manager = ctx.data().datalake_config.clone();
//     let search_query = Trivial::build_select_query(Some(&format!("id = {}", guild_id)), None, None);
//     let trivials = manager
//         .execute_select_with_retry::<Trivial>(&search_query)
//         .await;

//     if trivials.is_empty() {
//         return vec![];
//     }

//     trivials
//         .iter()
//         .map(|c| ChannelId::from(c.channel_id).to_string())
//         .collect()
// }
