use crate::helper::is_user_admin_right;
use crate::{Context, Error};
use clickhouse_pool::traits::Model;
use database::trivial::Trivial;
use poise::serenity_prelude;
use tracing::{debug, info, instrument};

/// Create a trivia game
/// Use the current channel by default, or specify a different channel.
#[instrument(name="trivia_create",skip(ctx), level = "info", fields(channel = ctx.channel_id().get(), guild = ?ctx.guild_id().unwrap().get()))]
#[poise::command(
    prefix_command,
    slash_command,
    guild_only,
    category = "Trivia",
    check = "is_user_admin_right"
)]
pub async fn create(
    ctx: Context<'_>,
    #[description = "Name of the trivia game"] name: String,
    #[description = "Description of the trivia game, by default empty"] description: Option<String>,
    #[description = "Channel to create the trivia game in, by default current channel"]
    channel: Option<serenity_prelude::GuildChannel>,
) -> Result<(), Error> {
    let guild_id = match ctx.guild_id() {
        Some(id) => id.get(),
        None => {
            ctx.say("This command can only be used in a server.")
                .await?;
            return Ok(());
        }
    };
    let channel_id = match channel {
        Some(c) => c.id.get(),
        None => {
            ctx.say("No channel specified, using current channel.")
                .await?;
            ctx.channel_id().get()
        }
    };
    let creator_id = ctx.author().id.get();
    let manager_pool = ctx.data().datalake_config.clone();

    // Check if a trivia game already exists in the channel
    let get_trivia = Trivial::build_select_query(
        Some(&format!(
            "channel_id = {} and guild_id = {}",
            channel_id, guild_id
        )),
        None,
        None,
    );

    let trivia_exists: Vec<Trivial> = manager_pool.execute_select_with_retry(&get_trivia).await?;
    if !trivia_exists.is_empty() {
        ctx.say(format!(
            "A trivia game already exists in this channel with the name: {}",
            trivia_exists[0].name
        ))
        .await?;
        return Ok(());
    }
    info!(
        "No existing trivia game found in channel <#{}>. Proceeding to create a new one.",
        channel_id
    );
    // Create a new trivia game
    let new_trivia = Trivial::new(
        name,
        description.unwrap_or_else(|| "".to_string()),
        guild_id,
        channel_id,
        creator_id,
        creator_id,
    );

    let insert_query = new_trivia.insert_query();
    debug!("Insert query: {}", insert_query);
    let mut inserter = match manager_pool
        .get_insert::<Trivial>(Trivial::table_name())
        .await
    {
        Ok(inserter) => inserter,
        Err(e) => {
            tracing::error!("Failed to create inserter for Trivial: {}", e);
            ctx.say("Failed to create trivia game. Please try again later.")
                .await?;
            return Ok(());
        }
    };
    inserter.write(&new_trivia).await?;
    match inserter.end().await {
        Ok(_) => {
            info!(
                "Trivia game '{}' created successfully in channel <#{}>.",
                new_trivia.name, channel_id
            );
            ctx.say(format!(
                "Trivia game '{}' created successfully in channel <#{}>.",
                new_trivia.name, channel_id
            ))
            .await?;
        }
        Err(e) => {
            tracing::error!("Failed to create trivia game: {}", e);
            ctx.say("Failed to create trivia game. Please try again later.")
                .await?;
        }
    }
    Ok(())
}
