use crate::helper::is_user_admin_right;
use crate::{Context, Error};
use clickhouse_pool::traits::Model;
use database::trivial::Trivial;
use poise::{
    serenity_prelude::{self, ComponentInteractionCollector},
    CreateReply,
};
use tracing::{info, instrument, warn};

/// Turn a trivia into an embed for listing
#[instrument(name = "trivia_list_embed", level = "info")]
pub fn trivia_list_embed(trivia: &Trivial, current_channel: bool) -> serenity_prelude::CreateEmbed {
    let mut embed = serenity_prelude::CreateEmbed::default()
        .title(&trivia.name)
        .description(&trivia.description)
        .field("Channel", format!("<#{}>", trivia.channel_id), true)
        .field("Status", format!("{:?}", trivia.status), true)
        .field("Reward Kind", format!("{:?}", trivia.reward_kind), true)
        .field("Reward Amount", trivia.reward_amount.to_string(), true);

    if current_channel {
        embed = embed.color(serenity_prelude::Color::GOLD);
    } else {
        embed = embed.color(serenity_prelude::Color::DARK_GREY);
    }

    embed
}

/// List trivia games in the current server and show if there are any in the current channel.
#[instrument(name="trivia_list",skip(ctx), level = "info", fields(channel = ctx.channel_id().get(), guild = ?ctx.guild_id().unwrap().get()))]
#[poise::command(
    prefix_command,
    slash_command,
    guild_only,
    category = "Trivia",
    check = "is_user_admin_right"
)]
pub async fn list(ctx: Context<'_>) -> Result<(), Error> {
    let guild_id = match ctx.guild_id() {
        Some(id) => id.get(),
        None => {
            ctx.say("This command can only be used in a server.")
                .await?;
            warn!("User tried to list trivia games in a DM channel.");
            return Ok(());
        }
    };
    let channel_id = ctx.channel_id().get();
    let manager_pool = ctx.data().datalake_config.clone();

    // Fetch all trivia games in the guild
    let get_trivia =
        Trivial::build_select_query(Some(&format!("guild_id = {}", guild_id)), None, None);

    let trivia_games: Vec<Trivial> = manager_pool.execute_select_with_retry(&get_trivia).await?;

    if trivia_games.is_empty() {
        ctx.say("No trivia games found in this server.").await?;
        info!(
            "User requested trivia list, but no games found in guild {}",
            guild_id
        );
        return Ok(());
    }

    // Put the current channel's trivia game in a vec
    let mut current_channel_trivia = Vec::new();
    for trivia in &trivia_games {
        if trivia.channel_id == channel_id {
            // push the trivia game to be the first one in the list
            current_channel_trivia.insert(0, trivia.clone());
        } else {
            current_channel_trivia.push(trivia.clone());
        }
    }

    if current_channel_trivia.len() == 1 {
        // If there is only one trivia game, just send it
        let reply = CreateReply::default().embed(trivia_list_embed(
            &current_channel_trivia[0],
            current_channel_trivia[0].channel_id == channel_id,
        ));
        ctx.send(reply).await?;
        return Ok(());
    }

    let ctx_id = ctx.id();
    let prev_button_id = format!("{}prev", ctx_id);
    let next_button_id = format!("{}next", ctx_id);
    let close_button_id = format!("{}close", ctx_id);

    let reply = {
        let components = serenity_prelude::CreateActionRow::Buttons(vec![
            serenity_prelude::CreateButton::new(&prev_button_id).emoji('◀'),
            serenity_prelude::CreateButton::new(&close_button_id)
                .emoji('❌')
                .style(serenity_prelude::ButtonStyle::Danger),
            serenity_prelude::CreateButton::new(&next_button_id).emoji('▶'),
        ]);
        CreateReply::default()
            .embed(trivia_list_embed(
                &current_channel_trivia[0],
                current_channel_trivia[0].channel_id == channel_id,
            ))
            .components(vec![components])
    };
    ctx.send(reply).await?;
    // Loop through incoming interactions with the navigation buttons
    let mut current_page = 0;
    while let Some(press) = ComponentInteractionCollector::new(ctx)
        .filter(move |press| press.data.custom_id.starts_with(&ctx_id.to_string()))
        .timeout(std::time::Duration::from_secs(3600 * 2))
        .await
    {
        if press.data.custom_id == next_button_id {
            current_page += 1;
            if current_page >= current_channel_trivia.len() {
                current_page = 0;
            }
        } else if press.data.custom_id == prev_button_id {
            current_page = current_page
                .checked_sub(1)
                .unwrap_or(current_channel_trivia.len() - 1);
        } else if press.data.custom_id == close_button_id {
            // Close the interaction
            press
                .create_response(
                    ctx.serenity_context(),
                    serenity_prelude::CreateInteractionResponse::UpdateMessage(
                        serenity_prelude::CreateInteractionResponseMessage::new()
                            .content("Trivia list closed.")
                            .components(vec![]),
                    ),
                )
                .await?;
            return Ok(());
        } else {
            // This is an unrelated button interaction
            continue;
        }

        // Update the message with the new page contents
        press
            .create_response(
                ctx.serenity_context(),
                serenity_prelude::CreateInteractionResponse::UpdateMessage(
                    serenity_prelude::CreateInteractionResponseMessage::new().embed(
                        trivia_list_embed(
                            &current_channel_trivia[current_page],
                            current_channel_trivia[current_page].channel_id == channel_id,
                        ),
                    ),
                ),
            )
            .await?;
    }
    Ok(())
}
