use chrono::{DateTime, Utc};
use clickhouse::Row;
use clickhouse_pool::traits::Model;
use serde::{Deserialize, Serialize};
use uuid::Uuid;

#[derive(Debug, Clone, PartialEq, PartialOrd, Row, Serialize, Deserialize)]
pub struct TrivialRound {
    #[serde(with = "clickhouse::serde::uuid")]
    pub id: Uuid,
    #[serde(with = "clickhouse::serde::uuid")]
    pub trivial_id: Uuid,
    #[serde(with = "clickhouse::serde::uuid")]
    pub question_id: Uuid,
    pub answer: Vec<(u64, String, DateTime<Utc>)>,

    pub finished: bool,

    #[serde(with = "clickhouse::serde::chrono::datetime64::millis")]
    pub created_at: DateTime<Utc>,
    #[serde(with = "clickhouse::serde::chrono::datetime64::millis")]
    pub updated_at: DateTime<Utc>,
}

impl Model for TrivialRound {
    type T = TrivialRound;

    fn table_name() -> &'static str {
        "trivial_round"
    }

    fn optimize_table_sql() -> &'static str {
        r#"
        OPTIMIZE TABLE trivial_round FINAL
        "#
        .trim()
    }

    fn create_table_sql() -> &'static str {
        r#"
        CREATE TABLE IF NOT EXISTS trivial_round (
            id UUID PRIMARY KEY,
            trivial_id UUID,
            question_id UUID,
            answer Array(Tuple(UInt64, String, DateTime64(3))),
            finished Bool,
            created_at DateTime64(3),
            updated_at DateTime64(3)
        ) ENGINE = MergeTree()
        ORDER BY (id, trivial_id)
        "#
        .trim()
    }

    fn column_names() -> Vec<&'static str> {
        vec![
            "id",
            "trivial_id",
            "question_id",
            "answer",
            "finished",
            "created_at",
            "updated_at",
        ]
    }

    fn to_row(&self) -> (Vec<&'static str>, Vec<String>) {
        (
            Self::column_names(),
            vec![
                self.id.to_string(),
                self.trivial_id.to_string(),
                self.question_id.to_string(),
                format!("{:?}", self.answer),
                self.finished.to_string(),
                self.created_at.to_string(),
                self.updated_at.to_string(),
            ],
        )
    }

    fn insert_query(&self) -> String {
        let (columns, values) = self.to_row();
        let columns_str = columns.join(", ");
        let values_str = values.join(", ");

        format!(
            "INSERT INTO {} ({}) VALUES ({})",
            Self::table_name(),
            columns_str,
            values_str
        )
    }

    fn batch_insert_query(items: &[Self::T]) -> String {
        let mut queries = Vec::new();
        for item in items {
            let (columns, values) = item.to_row();
            let columns_str = columns.join(", ");
            let values_str = values.join(", ");
            queries.push(format!(
                "INSERT INTO {} ({}) VALUES ({})",
                Self::table_name(),
                columns_str,
                values_str
            ));
        }
        queries.join("; ")
    }

    fn build_select_query(
        where_clause: Option<&str>,
        limit: Option<u64>,
        offset: Option<u64>,
    ) -> String {
        let mut query = format!("SELECT * FROM {}", Self::table_name());
        if let Some(where_clause) = where_clause {
            query.push_str(&format!(" WHERE {}", where_clause));
        }
        if let Some(limit) = limit {
            query.push_str(&format!(" LIMIT {}", limit));
        }
        if let Some(offset) = offset {
            query.push_str(&format!(" OFFSET {}", offset));
        }
        query
    }
}
